import os

import pytest

from leapp.libraries.actor import ntp2chrony

CUR_DIR = os.path.dirname(os.path.abspath(__file__))

NTP_CONF = os.path.join(CUR_DIR, "data/ntp.conf")
STEP_TICKERS = os.path.join(CUR_DIR, "data/step_tickers")

# TODO [Artem] the following consts should use abs path as well.
#   reader of [[:digit:]]chrony.conf files does not support wildcards, so we
#   have to change the working directory here for now.
NTP_MATCH_DIR = "data/ntpconfs/"
CHRONY_MATCH_DIR = "data/chronyconfs/"


@pytest.fixture
def adjust_cwd():
    previous_cwd = os.getcwd()
    os.chdir(CUR_DIR)
    yield
    os.chdir(previous_cwd)


class TestConverter(object):
    def test_basic(self):
        config = ntp2chrony.NtpConfiguration(CUR_DIR, NTP_CONF, step_tickers=STEP_TICKERS)
        present = [config.restrictions, config.driftfile, config.trusted_keys, config.keys,
                   config.step_tickers, config.restrictions]
        for section in present:
            assert section
        chrony_conf = config.get_chrony_conf('/etc/chrony.keys')
        # additional verification section by section for each param in present?

        # verify step_tickers -> initstepslew
        initstepslew_line = next((l for l in chrony_conf.split('\n')
                                  if l.startswith('initstepslew')), None)
        assert initstepslew_line and initstepslew_line.endswith(' '.join(config.step_tickers))
        chrony_keys = config.get_chrony_keys()
        # verify keys generation
        for num, _, key in config.keys:
            expected = ('%(num)s MD5 %(key)s' %
                        {'key': 'HEX:' if len(key) > 20 else 'ASCII:' + key, 'num': num})
            # keys not from trusted keys are commented out by default
            if not any(num in range(x, y + 1) for (x, y) in config.trusted_keys):
                expected = '#' + expected
            assert expected in chrony_keys


class TestConfigConversion(object):
    def _do_match(self, expected_file, actual):
        expected_lines = []
        actual_lines = []
        with open(expected_file) as f:
            expected_lines = [l.strip() for l in f.readlines()
                              if l.strip() and not l.strip().startswith('#')]
        actual_lines = [l.strip() for l in actual.split('\n')
                        if l.strip() and not l.strip().startswith('#')]
        assert expected_lines == actual_lines

    def _check_existance(self, fname, default=''):
        if os.path.exists(fname):
            return fname
        return default

    def test_match(self, adjust_cwd):

        for f in [fe for fe in os.listdir(NTP_MATCH_DIR) if fe.endswith('conf')]:
            # get recorded actual result
            num = f.split('.')[0].split('_')[0]
            ntp_conf = os.path.join(NTP_MATCH_DIR, f)
            step_tickers = self._check_existance(
                os.path.join(NTP_MATCH_DIR, '%s_step_tickers' % num))
            config = ntp2chrony.NtpConfiguration('',
                                                 ntp_conf,
                                                 step_tickers=step_tickers)
            potential_chrony_keys = os.path.join(CHRONY_MATCH_DIR, "%s_chrony.keys" % num)
            actual_data = config.get_chrony_conf(chrony_keys_path=potential_chrony_keys)
            expected_fname = os.path.join(CHRONY_MATCH_DIR, "%s_chrony.conf" % num)
            # make sure recorded and generated configs match
            self._do_match(expected_fname, actual_data)
            actual_keys = config.get_chrony_keys()
            expected_keys_file = self._check_existance(potential_chrony_keys)
            # if keys are recorded or generated make sure they match
            if actual_keys and expected_keys_file != '':
                self._do_match(expected_keys_file, actual_keys)



from leapp.actors import Actor
from leapp.libraries.actor import reportsettargetrelease
from leapp.models import Report
from leapp.tags import IPUWorkflowTag, TargetTransactionChecksPhaseTag


class ReportSetTargetRelease(Actor):
    """
    Reports information related to the release set in the subscription-manager after the upgrade.

    When using Red Hat subscription-manager (RHSM), the release is set by default
    to the target version release. In case of skipping the RHSM (--no-rhsm), the
    release stays as it was on the source RHEL and user has to handle it manually
    after the upgrade.
    """

    name = 'report_set_target_release'
    consumes = ()
    produces = (Report,)
    tags = (IPUWorkflowTag, TargetTransactionChecksPhaseTag)

    def process(self):
        reportsettargetrelease.process()



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class FirewallStatus(Model):
    topic = SystemFactsTopic

    enabled = fields.Boolean()
    active = fields.Boolean()


class FirewallsFacts(Model):
    topic = SystemFactsTopic

    firewalld = fields.Model(FirewallStatus)
    iptables = fields.Model(FirewallStatus)
    ip6tables = fields.Model(FirewallStatus)



from leapp.actors import Actor
from leapp.libraries.actor import add_provider
from leapp.models import OpenSslConfig
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag


class OpenSslProviders(Actor):
    """
    Modify the openssl.cnf file to support new providers in OpenSSL 3.0

    Change the initialization:

    - openssl_conf = default_modules
    + openssl_conf = openssl_init

    Rename the default block and link the providers block:

    - [default_modules]
    + [openssl_init]
    + providers = provider_sect

    Add the providers block:

    + [provider_sect]
    + default = default_sect
    + ##legacy = legacy_sect
    +
    + [default_sect]
    + activate = 1
    +
    + ##[legacy_sect]
    + ##activate = 1
    """

    name = 'open_ssl_providers'
    consumes = (OpenSslConfig,)
    produces = ()
    tags = (IPUWorkflowTag, ApplicationsPhaseTag,)

    def process(self):
        add_provider.process(self.consume(OpenSslConfig))



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class SelinuxRelabelDecision(Model):
    topic = SystemInfoTopic

    set_relabel = fields.Boolean()


class SelinuxPermissiveDecision(Model):
    topic = SystemInfoTopic

    set_permissive = fields.Boolean()



from leapp.models import FirewalldGlobalConfig, FirewallsFacts, FirewallStatus
from leapp.reporting import Report


def test_actor_firewalldcheckallowzonedrifting(current_actor_context):
    status = FirewallStatus(enabled=True, active=True)
    current_actor_context.feed(FirewallsFacts(firewalld=status,
                                              iptables=status,
                                              ip6tables=status))
    current_actor_context.feed(FirewalldGlobalConfig(allowzonedrifting=True))

    current_actor_context.run()
    assert current_actor_context.consume(Report)


def test_actor_firewalldcheckallowzonedrifting_negative(current_actor_context):
    status = FirewallStatus(enabled=False, active=True)
    current_actor_context.feed(FirewallsFacts(firewalld=status,
                                              iptables=status,
                                              ip6tables=status))
    current_actor_context.feed(FirewalldGlobalConfig(allowzonedrifting=True))

    current_actor_context.run()
    assert not current_actor_context.consume(Report)


def test_actor_firewalldcheckallowzonedrifting_negative2(current_actor_context):
    status = FirewallStatus(enabled=True, active=True)
    current_actor_context.feed(FirewallsFacts(firewalld=status,
                                              iptables=status,
                                              ip6tables=status))
    current_actor_context.feed(FirewalldGlobalConfig(allowzonedrifting=False))

    current_actor_context.run()
    assert not current_actor_context.consume(Report)



from leapp.actors import Actor
from leapp.libraries.actor import checkremovedenvvars
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckRemovedEnvVars(Actor):
    """
    Check for usage of removed environment variables and inhibit the upgrade
    if they are used.
    """

    name = 'check_removed_envvars'
    consumes = ()
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        checkremovedenvvars.process()



import os.path

from leapp.models import DNFWorkaround


def test_register_ruby_irb_adjustments(current_actor_context):
    current_actor_context.run()
    assert len(current_actor_context.consume(DNFWorkaround)) == 1
    assert current_actor_context.consume(DNFWorkaround)[0].display_name == 'IRB directory fix'
    assert os.path.basename(current_actor_context.consume(DNFWorkaround)[0].script_path) == 'handlerubyirbsymlink'
    assert os.path.exists(current_actor_context.consume(DNFWorkaround)[0].script_path)



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class PartitionEntry(Model):
    topic = SystemInfoTopic

    major = fields.String()
    minor = fields.String()
    blocks = fields.String()
    name = fields.String()


class FstabEntry(Model):
    topic = SystemInfoTopic

    fs_spec = fields.String()
    fs_file = fields.String()
    fs_vfstype = fields.String()
    fs_mntops = fields.String()
    fs_freq = fields.String()
    fs_passno = fields.String()


class MountEntry(Model):
    topic = SystemInfoTopic

    name = fields.String()
    mount = fields.String()
    tp = fields.String()
    options = fields.String()


class LsblkEntry(Model):
    topic = SystemInfoTopic

    name = fields.String()
    kname = fields.String()
    maj_min = fields.String()
    rm = fields.String()
    size = fields.String()
    bsize = fields.Integer()
    ro = fields.String()
    tp = fields.String()
    mountpoint = fields.String()


class PvsEntry(Model):
    topic = SystemInfoTopic

    pv = fields.String()
    vg = fields.String()
    fmt = fields.String()
    attr = fields.String()
    psize = fields.String()
    pfree = fields.String()


class VgsEntry(Model):
    topic = SystemInfoTopic

    vg = fields.String()
    pv = fields.String()
    lv = fields.String()
    sn = fields.String()
    attr = fields.String()
    vsize = fields.String()
    vfree = fields.String()


class LvdisplayEntry(Model):
    topic = SystemInfoTopic

    lv = fields.String()
    vg = fields.String()
    attr = fields.String()
    lsize = fields.String()
    pool = fields.String()
    origin = fields.String()
    data = fields.String()
    meta = fields.String()
    move = fields.String()
    log = fields.String()
    cpy_sync = fields.String()
    convert = fields.String()


class SystemdMountEntry(Model):
    topic = SystemInfoTopic

    node = fields.String()
    path = fields.String()
    model = fields.String()
    wwn = fields.String()
    fs_type = fields.String()
    label = fields.String()
    uuid = fields.String()


class StorageInfo(Model):
    topic = SystemInfoTopic
    partitions = fields.List(fields.Model(PartitionEntry), default=[])
    fstab = fields.List(fields.Model(FstabEntry), default=[])
    mount = fields.List(fields.Model(MountEntry), default=[])
    lsblk = fields.List(fields.Model(LsblkEntry), default=[])
    pvs = fields.List(fields.Model(PvsEntry), default=[])
    vgs = fields.List(fields.Model(VgsEntry), default=[])
    lvdisplay = fields.List(fields.Model(LvdisplayEntry), default=[])
    systemdmount = fields.List(fields.Model(SystemdMountEntry), default=[])



from leapp import reporting
from leapp.actors import Actor
from leapp.libraries.actor import checksendmail
from leapp.libraries.common.rpms import has_package
from leapp.libraries.common.tcpwrappersutils import config_applies_to_daemon
from leapp.models import DistributionSignedRPM, SendmailMigrationDecision, TcpWrappersFacts
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag

COMMON_REPORT_TAGS = [reporting.Groups.SERVICES, reporting.Groups.EMAIL]

related = [
              reporting.RelatedResource('file', f) for f in checksendmail.get_conf_files()
          ] + [reporting.RelatedResource('package', 'sendmail')]


class CheckSendmail(Actor):
    """
    Check if sendmail is installed, check whether configuration update is needed, inhibit upgrade if TCP wrappers
    are used.
    """

    name = 'check_sendmail'
    consumes = (DistributionSignedRPM, TcpWrappersFacts,)
    produces = (Report, SendmailMigrationDecision,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        if not has_package(DistributionSignedRPM, 'sendmail'):
            return

        if config_applies_to_daemon(next(self.consume(TcpWrappersFacts)), 'sendmail'):
            create_report([
                reporting.Title('TCP wrappers support removed in the next major version'),
                reporting.Summary(
                    'TCP wrappers are legacy host-based ACL (Access Control List) system '
                    'which has been removed in the next major version of RHEL.'
                ),
                reporting.Remediation(
                    hint='Please migrate from TCP wrappers to some other access control mechanism and delete '
                         'sendmail from the /etc/hosts.[allow|deny].'
                ),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Groups(COMMON_REPORT_TAGS + [reporting.Groups.NETWORK]),
                reporting.Groups([reporting.Groups.INHIBITOR])
            ] + related)

            return
        migrate_files = checksendmail.check_files_for_compressed_ipv6()
        if migrate_files:
            create_report([
                reporting.Title('sendmail configuration will be migrated'),
                reporting.Summary(
                    'IPv6 addresses will be uncompressed, check all IPv6 addresses in all sendmail '
                    'configuration files for correctness.'
                ),
                reporting.Severity(reporting.Severity.LOW),
                reporting.Groups(COMMON_REPORT_TAGS)
            ] + related)

            self.produce(SendmailMigrationDecision(migrate_files=migrate_files))
        else:
            self.log.info('The sendmail configuration seems compatible - it won\'t be migrated.')



from leapp.actors import Actor
from leapp.libraries.actor import cupscheck
from leapp.models import CupsChangedFeatures, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CupsCheck(Actor):
    """
    Reports changes in configuration between CUPS 1.6.3 and 2.2.6

    Reports if user configuration contains features (interface scripts),
    directives (Include, PrintcapFormat, PassEnv, SetEnv,
    ServerCertificate, ServerKey) or directive values (Digest,
    BasicDigest). Some of them were removed for security reasons
    (interface scripts and directive Include), moved
    to cups-files.conf for security reasons (PassEnv, SetEnv).
    Others were removed (ServerCertificate, ServerKey, Digest,
    BasicDigest) or moved (PrintcapFormat) due deprecation.
    """

    name = 'cups_check'
    consumes = (CupsChangedFeatures,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        cupscheck.make_reports()



from leapp import reporting
from leapp.actors import Actor
from leapp.libraries.common.rpms import has_package
from leapp.models import DistributionSignedRPM, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag

UNSUPPORTED_VERSIONS = ['2.1', '3.0', '3.1', '5.0']


class DotnetUnsupportedVersionsCheck(Actor):
    """
    Check for installed .NET versions that are no longer supported.
    """

    name = 'dotnet_unsupported_versions_check'
    consumes = (DistributionSignedRPM,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        unsupported_versions_report_text = ''

        for unsupported_version in UNSUPPORTED_VERSIONS:
            runtime_package = f'dotnet-runtime-{unsupported_version}'
            if has_package(DistributionSignedRPM, runtime_package):
                unsupported_versions_report_text += '{0}{1}'.format('\n    - ', unsupported_version)

        if unsupported_versions_report_text:
            reporting.create_report([
                reporting.Title('Unsupported .NET versions installed on the system.'),
                reporting.Summary(
                    (
                        'The following versions of .NET are no longer supported :{0}\n'
                        'Applications that use these runtimes will no longer work\n'
                        'and must be updated to target a newer version of .NET.'
                    ).format(
                        unsupported_versions_report_text
                    )
                ),
                reporting.Severity(reporting.Severity.HIGH)])



from leapp.actors import Actor
from leapp.libraries.actor import spamassassinconfigcheck
from leapp.models import Report, SpamassassinFacts
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class SpamassassinConfigCheck(Actor):
    """
    Reports changes in spamassassin between RHEL-7 and RHEL-8

    Reports backward-incompatible changes that have been made in spamassassin
    between RHEL-7 and RHEL-8 (spamc no longer accepts an argument with the --ssl
    option; spamd no longer accepts the --ssl-version; SSLv3 is no longer supported;
    the type of spamassassin.service has been changed from "forking" to "simple";
    sa-update no longer supports SHA1 validation of rule files).

    The migration of the configuration files will be mostly handled by the
    SpamassassinConfigUpdate actor, however the admin still needs to know about
    the changes so that they can do any necessary migration in places that we cannot
    reach (e.g. scripts).
    """

    name = 'spamassassin_config_check'
    consumes = (SpamassassinFacts,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        facts = next(self.consume(SpamassassinFacts), None)
        if facts:
            spamassassinconfigcheck.produce_reports(facts)
        else:
            self.log.debug('Skipping execution - no SpamassassinFacts message has been produced.')



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class IpaInfo(Model):
    topic = SystemFactsTopic

    has_client_package = fields.Boolean()
    is_client_configured = fields.Boolean()

    has_server_package = fields.Boolean()
    is_server_configured = fields.Boolean()



from leapp.actors import Actor
from leapp.libraries.actor.removeupgradebootentry import remove_boot_entry
from leapp.models import BootContent, FirmwareFacts
from leapp.tags import InitRamStartPhaseTag, IPUWorkflowTag


class RemoveUpgradeBootEntry(Actor):
    """
    Remove boot entry for Leapp provided initramfs.

    After upgrade process used initramfs after reboot, this entry is not necessary anymore.
    """

    name = 'remove_upgrade_boot_entry'
    consumes = (BootContent, FirmwareFacts)
    produces = ()
    tags = (IPUWorkflowTag, InitRamStartPhaseTag)

    def process(self):
        remove_boot_entry()



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class GrubCfgBios(Model):
    """
    A message providing info about BIOS (non-EFI) GRUB config
    """
    topic = SystemFactsTopic

    insmod_bls = fields.Boolean()
    """ Inform whether blscfg is loaded """



from leapp.models import InstalledDesktopsFacts, InstalledRPM, RPM
from leapp.snactor.fixture import current_actor_context

RH_PACKAGER = 'Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>'
Gnome_RPM = RPM(name='gnome-session', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
                pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51')
KDE_RPM = RPM(name='kde-workspace', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
              pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51')


def test_Gnome_present(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[Gnome_RPM, ]))
    current_actor_context.run()
    message = current_actor_context.consume(InstalledDesktopsFacts)[0]
    assert message.gnome_installed


def test_KDE_present(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[KDE_RPM, ]))
    current_actor_context.run()
    message = current_actor_context.consume(InstalledDesktopsFacts)[0]
    assert message.kde_installed


def test_KDE_Gnome_present(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[Gnome_RPM, KDE_RPM]))
    current_actor_context.run()
    message = current_actor_context.consume(InstalledDesktopsFacts)[0]
    assert message.gnome_installed and message.kde_installed


def test_no_desktop_present(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[]))
    current_actor_context.run()
    message = current_actor_context.consume(InstalledDesktopsFacts)[0]
    assert not message.gnome_installed and not message.kde_installed



from leapp.models import fields, Model
from leapp.topics import RHSMTopic


class RHSMInfo(Model):
    """
    Subscription-manager details required for the inplace upgrade.
    """
    topic = RHSMTopic

    release = fields.Nullable(fields.String())
    """ Release the subscription-manager is set to. """
    attached_skus = fields.List(fields.String(), default=[])
    """ SKUs the current system is attached to. """
    available_repos = fields.List(fields.String(), default=[])
    """ Repositories that are available to the current system through the subscription-manager. """
    enabled_repos = fields.List(fields.String(), default=[])
    """ Repositories that are enabled on the current system through the subscription-manager. """
    existing_product_certificates = fields.List(fields.String(), default=[])
    """ Product certificates that are currently installed on the system. """
    sca_detected = fields.Boolean(default=False)
    """ Info about whether SCA manifest was used or not. """



from leapp.actors import Actor
from leapp.libraries.actor import tcpwrappersconfigread
from leapp.models import TcpWrappersFacts
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class TcpWrappersConfigRead(Actor):
    """
    Parse tcp_wrappers configuration files /etc/hosts.{allow,deny}.
    """

    name = 'tcp_wrappers_config_read'
    consumes = ()
    produces = (TcpWrappersFacts,)
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        self.produce(tcpwrappersconfigread.get_tcp_wrappers_facts())



from collections import namedtuple

import pytest

from leapp.exceptions import RequestStopAfterPhase
from leapp.libraries.actor import verifycheckresults
from leapp.libraries.stdlib import api


class Report(object):
    def __init__(self, message):
        self.message = message

    def get(self, _, __):
        if self.message == 'title_with_inhibitor':
            return ['inhibitor']
        return []

    def __getitem__(self, _):
        return self.message


def test_actor(monkeypatch):
    def report_mocked(*models):
        yield namedtuple('msg', ['report'])(Report('title_with_inhibitor'))

    monkeypatch.setattr(api, "consume", report_mocked)

    with pytest.raises(RequestStopAfterPhase):
        verifycheckresults.check()


def test_actor_no_inhibitor(monkeypatch):
    def report_mocked(*models):
        yield namedtuple('msg', ['report'])(Report('title_without_inhibitor'))

    monkeypatch.setattr(api, "consume", report_mocked)
    verifycheckresults.check()



from leapp.libraries.actor.cryptopoliciescheck import _get_files_to_copy
from leapp.models import CryptoPolicyInfo, CustomCryptoPolicy, CustomCryptoPolicyModule


def test_get_files_to_copy():
    cpi = CryptoPolicyInfo(current_policy="DEFAULT", custom_policies=[], custom_modules=[])
    assert _get_files_to_copy(cpi) == []

    cpi.custom_policies.append(CustomCryptoPolicy(name="CUSTOM", path="/path/to/CUSTOM.pol"))
    assert _get_files_to_copy(cpi) == ["/path/to/CUSTOM.pol"]

    cpi.custom_modules.append(CustomCryptoPolicyModule(name="FIX", path="/path/to/FIX.mpol"))
    assert _get_files_to_copy(cpi) == ["/path/to/CUSTOM.pol", "/path/to/FIX.mpol"]




from leapp.libraries.actor import setuptargetrepos_repomap
from leapp.libraries.common.config.version import get_source_major_version
from leapp.libraries.stdlib import api
from leapp.models import (
    CustomTargetRepository,
    InstalledRPM,
    RepositoriesBlacklisted,
    RepositoriesFacts,
    RepositoriesMapping,
    RepositoriesSetupTasks,
    RHELTargetRepository,
    RHUIInfo,
    SkippedRepositories,
    TargetRepositories,
    UsedRepositories
)


def _get_enabled_repoids():
    """
    Collects repoids of all enabled repositories on the source system.

    :returns: Set of all enabled repository IDs present on the source system.
    :rtype: Set[str]
    """
    enabled_repoids = set()
    for repos in api.consume(RepositoriesFacts):
        for repo_file in repos.repositories:
            for repo in repo_file.data:
                if repo.enabled:
                    enabled_repoids.add(repo.repoid)
    return enabled_repoids


def _get_repoids_from_installed_packages():
    repoids_from_installed_packages = set()
    for installed_packages in api.consume(InstalledRPM):
        for rpm_package in installed_packages.items:
            repoids_from_installed_packages.add(rpm_package.repository)
    return repoids_from_installed_packages


def _get_blacklisted_repoids():
    repos_blacklisted = set()
    for blacklist in api.consume(RepositoriesBlacklisted):
        repos_blacklisted.update(blacklist.repoids)
    return repos_blacklisted


def _get_custom_target_repos():
    custom_repos = []
    for repo in api.consume(CustomTargetRepository):
        custom_repos.append(repo)
    return custom_repos


def _get_used_repo_dict():
    """
    Return dict: {used_repoid: [installed_packages]}
    """
    used = {}
    for used_repos in api.consume(UsedRepositories):
        for used_repo in used_repos.repositories:
            used[used_repo.repository] = used_repo.packages
    return used


def _get_mapped_repoids(repomap, src_repoids):
    mapped_repoids = set()
    src_maj_ver = get_source_major_version()
    for repoid in src_repoids:
        if repomap.get_pesid_repo_entry(repoid, src_maj_ver):
            mapped_repoids.add(repoid)
    return mapped_repoids


def process():
    # Load relevant data from messages
    used_repoids_dict = _get_used_repo_dict()
    enabled_repoids = _get_enabled_repoids()
    excluded_repoids = _get_blacklisted_repoids()
    custom_repos = _get_custom_target_repos()
    repoids_from_installed_packages = _get_repoids_from_installed_packages()

    # Setup repomap handler
    repo_mappig_msg = next(api.consume(RepositoriesMapping), RepositoriesMapping())

    rhui_info = next(api.consume(RHUIInfo), None)
    cloud_provider = rhui_info.provider if rhui_info else ''

    repomap = setuptargetrepos_repomap.RepoMapDataHandler(repo_mappig_msg, cloud_provider=cloud_provider)

    # Filter set of repoids from installed packages so that it contains only repoids with mapping
    repoids_from_installed_packages_with_mapping = _get_mapped_repoids(repomap, repoids_from_installed_packages)

    # Set of repoid that are going to be mapped to target repoids containing enabled repoids and also repoids from
    # installed packages that have mapping to prevent missing repositories that are disabled during the upgrade, but
    # can be used to upgrade installed packages.
    repoids_to_map = enabled_repoids.union(repoids_from_installed_packages_with_mapping)

    # Set default repository channels for the repomap
    # TODO(pstodulk): what about skip this completely and keep the default 'ga'..?
    default_channels = setuptargetrepos_repomap.get_default_repository_channels(repomap, repoids_to_map)
    repomap.set_default_channels(default_channels)

    # Get target RHEL repoids based on the repomap
    expected_repos = repomap.get_expected_target_pesid_repos(repoids_to_map)
    target_rhel_repoids = set()
    for target_pesid, target_pesidrepo in expected_repos.items():
        if not target_pesidrepo:
            # NOTE this could happen only for enabled repositories part of the set,
            # since the repositories collected from installed packages already contain
            # only mappable repoids.

            # With the original repomap data, this should not happen (this should
            # currently point to a problem in our data
            # TODO(pstodulk): add report? inhibitor? what should be in the report?
            api.current_logger().error(
                'Missing target repository from the {} family (PES ID).'
                .format(target_pesid)
            )
            continue
        if target_pesidrepo.repoid in excluded_repoids:
            api.current_logger().debug('Skipping the {} repo (excluded).'.format(target_pesidrepo.repoid))
            continue
        target_rhel_repoids.add(target_pesidrepo.repoid)

    # FIXME: this could possibly result into a try to enable multiple repositories
    # from the same family (pesid). But unless we have a bug in previous actors,
    # it should not happen :) it's not blocker error anyway, so survive it.
    # - We expect to deliver the fix as part of the refactoring when we merge
    # setuptargetrepos & peseventsscanner actors together (+ blacklistrepos?)
    for task in api.consume(RepositoriesSetupTasks):
        for repo in task.to_enable:
            if repo in excluded_repoids:
                api.current_logger().debug('Skipping the {} repo from setup task (excluded).'.format(repo))
                continue
            target_rhel_repoids.add(repo)

    # create the final lists and sort them (for easier testing)
    rhel_repos = [RHELTargetRepository(repoid=repoid) for repoid in sorted(target_rhel_repoids)]
    custom_repos = [repo for repo in custom_repos if repo.repoid not in excluded_repoids]
    custom_repos = sorted(custom_repos, key=lambda x: x.repoid)

    # produce message about skipped repositories
    enabled_repoids_with_mapping = _get_mapped_repoids(repomap, enabled_repoids)
    skipped_repoids = enabled_repoids & set(used_repoids_dict.keys()) - enabled_repoids_with_mapping
    if skipped_repoids:
        pkgs = set()
        for repo in skipped_repoids:
            pkgs.update(used_repoids_dict[repo])
        api.produce(SkippedRepositories(repos=sorted(skipped_repoids), packages=sorted(pkgs)))

    api.produce(TargetRepositories(
        rhel_repos=rhel_repos,
        custom_repos=custom_repos,
    ))



import pytest

from leapp import reporting
from leapp.libraries.actor import checksystemdservicetasks
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import SystemdServicesTasks
from leapp.utils.report import is_inhibitor


@pytest.mark.parametrize(
    ('tasks', 'should_inhibit'),
    [
        (
            [SystemdServicesTasks(to_enable=['hello.service'], to_disable=['hello.service'])],
            True
        ),
        (
            [SystemdServicesTasks(to_enable=['hello.service', 'world.service'],
                                  to_disable=['hello.service'])],
            True
        ),
        (
            [
                SystemdServicesTasks(to_enable=['hello.service']),
                SystemdServicesTasks(to_disable=['hello.service'])
            ],
            True
        ),
        (
            [SystemdServicesTasks(to_enable=['hello.service'], to_disable=['world.service'])],
            False
        ),
        (
            [
                SystemdServicesTasks(to_enable=['hello.service']),
                SystemdServicesTasks(to_disable=['world.service'])
            ],
            False
        ),
        (
            [
                SystemdServicesTasks(to_enable=['hello.service', 'world.service']),
                SystemdServicesTasks(to_disable=['world.service', 'httpd.service'])
            ],
            True
        ),
        (
            [
                SystemdServicesTasks(to_enable=['hello.service']),
                SystemdServicesTasks(to_disable=['world.service']),
                SystemdServicesTasks(to_enable=['hello.service', 'kitty.service'])
            ],
            False
        ),
        (
            [],
            False
        )
    ]
)
def test_conflicts_detected(monkeypatch, tasks, should_inhibit):

    created_reports = create_report_mocked()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=tasks))
    monkeypatch.setattr(reporting, 'create_report', created_reports)

    checksystemdservicetasks.check_conflicts()

    assert bool(created_reports.called) == should_inhibit
    assert is_inhibitor(created_reports.report_fields) == should_inhibit


@pytest.mark.parametrize(
    ('tasks', 'expected_reported'),
    [
        (
            [SystemdServicesTasks(to_enable=['world.service', 'httpd.service', 'hello.service'],
                                  to_disable=['hello.service', 'world.service', 'test.service'])],
            ['world.service', 'hello.service']
        ),
        (
            [
                SystemdServicesTasks(to_enable=['hello.service', 'httpd.service'],
                                     to_disable=['world.service']),
                SystemdServicesTasks(to_enable=['world.service', 'httpd.service'],
                                     to_disable=['hello.service', 'test.service'])
            ],
            ['world.service', 'hello.service']
        ),
    ]
)
def test_coflict_reported(monkeypatch, tasks, expected_reported):

    created_reports = create_report_mocked()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=tasks))
    monkeypatch.setattr(reporting, 'create_report', created_reports)

    checksystemdservicetasks.check_conflicts()

    report_summary = created_reports.report_fields['summary']
    assert all(service in report_summary for service in expected_reported)



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class BootEntry(Model):
    """
    One entry in the boot loader configuration.

    Not meant to be produced directly, only as a part of :class:`SourceBootLoaderConfiguration`.
    """
    topic = SystemInfoTopic

    title = fields.String()
    """Title of the boot entry."""

    kernel_image = fields.String()
    """Kernel image of the boot entry."""


class SourceBootLoaderConfiguration(Model):
    """Describes the bootloader configuration found on the source system."""
    topic = SystemInfoTopic

    entries = fields.List(fields.Model(BootEntry))
    """Boot entries available in the bootloader configuration."""



from leapp.actors import Actor
from leapp.libraries.actor.checkcifs import checkcifs
from leapp.models import StorageInfo
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckCIFS(Actor):
    """
    Check if CIFS filesystem is in use. If yes, inhibit the upgrade process.

    Actor looks for CIFS in /ets/fstab.
    If there is a CIFS entry, the upgrade is inhibited.
    """
    name = "check_cifs"
    consumes = (StorageInfo,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag,)

    def process(self):
        checkcifs(self.consume(StorageInfo))



from leapp import reporting
from leapp.libraries.common import isccfg
from leapp.libraries.stdlib import api
from leapp.models import BindConfigIssuesModel, BindFacts


def add_statement(statement, state):
    """Add searched statement to found issues."""

    stmt_text = statement.serialize_skip(' ')
    name = statement.var(0).value()
    if name in state:
        state[name].append((stmt_text, statement.config.path))
    else:
        state[name] = [(stmt_text, statement.config.path)]


def find_dnssec_lookaside(statement, state):
    try:
        arg = statement.var(1)
        if not (arg.type() == arg.TYPE_BARE and arg.value() == 'no'):
            # auto or yes statement
            # dnssec-lookaside "." trust-anchor "dlv.isc.org";
            add_statement(statement, state)
    except IndexError:
        api.current_logger().warning('Unexpected statement format: "%s"',
                                     statement.serialize_skip(' '))


def convert_to_issues(statements):
    """Produce list of offending statements in set of files.

    :param statements: one item from list created by add_statement
    """

    files = dict()
    for statement, path in statements:
        if path in files:
            files[path].update(statement)
            if statement not in files[path].statements:
                files[path].statements.append(statement)
        else:
            files[path] = set(statement)
    values = list()
    for path in files:
        values.append(BindConfigIssuesModel(path=path, statements=list(files[path])))
    return values


def convert_found_state(state, files):
    """Convert find state results to facts.

    Check found statements and create facts from them."""

    dnssec_lookaside = None
    if 'dnssec-lookaside' in state:
        dnssec_lookaside = convert_to_issues(state['dnssec-lookaside'])

    return BindFacts(config_files=files,
                     modified_files=[],
                     dnssec_lookaside=dnssec_lookaside,
                     listen_on_v6_missing='listen-on-v6' not in state)


def get_facts(path, log=None):
    """Find issues in configuration files.

    Report used configuration files and wrong statements in each file.
    """

    find_calls = {
        'dnssec-lookaside': find_dnssec_lookaside,
        'listen-on-v6': add_statement,
    }

    parser = isccfg.IscConfigParser(path)
    state = {}
    files = set()

    for cfg in parser.FILES_TO_CHECK:
        parser.walk(cfg.root_section(), find_calls, state)
        files.add(cfg.path)

        api.current_logger().debug('Found state: "%s", files: "%s"',
                                   repr(state), files)

    facts = convert_found_state(state, list(files))
    return facts


def make_report(facts):
    """Make report message from gathered facts."""
    summary_messages = []
    report = []
    if facts.dnssec_lookaside:
        summary_messages.append('BIND configuration contains no longer accepted statements: dnssec-lookaside.')
    if facts.listen_on_v6_missing:
        summary_messages.append('Default value of listen-on-v6 have changed, but it is not present in configuration.'
                                ' named service will now listen on INET6 sockets also.')

    if summary_messages:
        summary = ' '.join(summary_messages)
        report.extend([
            reporting.Title('BIND configuration issues found'),
            reporting.Summary(summary),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.SERVICES, reporting.Groups.NETWORK]),
        ])

    return report



from leapp import reporting
from leapp.actors import Actor
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api
from leapp.models import PamConfiguration, Report
from leapp.reporting import create_report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckRemovedPamModules(Actor):
    """
    Check for modules that are not available in RHEL 8 anymore

    At this moment, we check only for pam_tally2. Few more modules
    are already covered in RemoveOldPAMModulesApply actor
    """

    name = 'removed_pam_modules'
    consumes = (PamConfiguration, )
    produces = (Report, )
    tags = (ChecksPhaseTag, IPUWorkflowTag, )

    def process(self):
        messages = self.consume(PamConfiguration)
        config = next(messages, None)
        if list(messages):
            api.current_logger().warning('Unexpectedly received more than one PamConfiguration message.')
        if not config:
            raise StopActorExecutionError(
                'Could not check pam configuration', details={'details': 'No PamConfiguration facts found.'}
            )

        # This list contain tuples of removed modules and their recommended replacements
        removed_modules = [
            ('pam_tally2', 'pam_faillock'),
            ]
        found_services = set()
        found_modules = set()
        replacements = set()
        for service in config.services:
            for module in removed_modules:
                removed = module[0]
                replacement = module[1]
                if removed in service.modules:
                    found_services.add(service.service)
                    found_modules.add(removed)
                    replacements.add(replacement)

        if found_modules:
            create_report([
                reporting.Title('The {} pam module(s) no longer available'.format(', '.join(found_modules))),
                reporting.Summary('The services {} using PAM are configured to '
                                  'use {} module(s), which is no longer available '
                                  'in Red Hat Enterprise Linux 8.'.format(
                                      ', '.join(found_services), ', '.join(found_modules))),
                reporting.Remediation(
                    hint='If you depend on its functionality, it is '
                         'recommended to migrate to {}. Otherwise '
                         'please remove the pam module(s) from all the files '
                         'under /etc/pam.d/.'.format(', '.join(replacements))
                ),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Groups([reporting.Groups.INHIBITOR]),
            ] + [reporting.RelatedResource('pam', r) for r in replacements | found_modules])



from leapp import reporting
from leapp.actors import Actor
from leapp.libraries.actor import checkbrltty
from leapp.libraries.common.rpms import has_package
from leapp.models import BrlttyMigrationDecision, DistributionSignedRPM
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag

related = [reporting.RelatedResource('package', 'brltty')]


class CheckBrltty(Actor):
    """
    Check if brltty is installed, check whether configuration update is needed.
    """

    name = 'check_brltty'
    consumes = (DistributionSignedRPM,)
    produces = (Report, BrlttyMigrationDecision,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        if has_package(DistributionSignedRPM, 'brltty'):
            create_report([
                reporting.Title('Brltty has incompatible changes in the next major version'),
                reporting.Summary(
                    'The --message-delay brltty option has been renamed to --message-timeout.\n'
                    'The -U [--update-interval=] brltty option has been removed.'
                ),
                reporting.Severity(reporting.Severity.LOW),
                reporting.Groups([reporting.Groups.ACCESSIBILITY]),
                reporting.Remediation(
                    hint='Please update your scripts to be compatible with the changes.'
                )
            ] + related)

            (migrate_file, migrate_bt, migrate_espeak,) = checkbrltty.check_for_unsupported_cfg()
            report_summary = ''
            if migrate_bt:
                report_summary = 'Unsupported aliases for bluetooth devices (\'bth:\' and \'bluez:\') will be '
                report_summary += 'renamed to \'bluetooth:\'.'
            if migrate_espeak:
                if report_summary:
                    report_summary += '\n'
                report_summary += 'eSpeak speech driver is no longer supported, it will be switched to eSpeak-NG.'
            if report_summary:
                create_report([
                    reporting.Title('brltty configuration will be migrated'),
                    reporting.Summary(report_summary),
                    reporting.Severity(reporting.Severity.LOW),
                    reporting.Groups([reporting.Groups.ACCESSIBILITY]),
                ] + related)

                self.produce(BrlttyMigrationDecision(migrate_file=migrate_file, migrate_bt=migrate_bt,
                                                     migrate_espeak=migrate_espeak))



import io  # Python2/Python3 compatible IO (open etc.)
import json
import os

import requests

from leapp import models
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common.config import get_consumed_data_stream_id, get_env
from leapp.libraries.common.rpms import get_leapp_packages, LeappComponents
from leapp.libraries.stdlib import api

SERVICE_HOST_DEFAULT = "https://cert.cloud.redhat.com"
REQUEST_TIMEOUT = (5, 30)
MAX_ATTEMPTS = 3
ASSET_PROVIDED_DATA_STREAMS_FIELD = 'provided_data_streams'


def _get_hint(local_path):
    hint = (
        'All official data files are part of the installed rpms these days.'
        ' The rpm is the only official source of the official data files for in-place upgrades.'
        ' This issue is usually encountered when the data files are incorrectly customized, replaced, or removed'
        ' (e.g. by custom scripts).'
        ' In case you want to recover the original {lp} file, remove the current one (if it still exists)'
        ' and reinstall the following packages: {rpms}.'
        .format(
            lp=local_path,
            rpms=', '.join(get_leapp_packages(component=LeappComponents.REPOSITORY))
        )
    )
    return hint


def _raise_error(local_path, details):
    """
    If the file acquisition fails in any way, throw an informative error to stop the actor.
    """
    summary = 'Data file {lp} is missing or invalid.'.format(lp=local_path)

    raise StopActorExecutionError(summary, details={'details': details, 'hint': _get_hint(local_path)})


def _request_data(service_path, cert, proxies, timeout=REQUEST_TIMEOUT):
    logger = api.current_logger()
    attempt = 0
    while True:
        attempt += 1
        try:
            return requests.get(service_path, cert=cert, proxies=proxies, timeout=REQUEST_TIMEOUT)
        except requests.exceptions.Timeout as e:
            etype_msg = 'Connection timeout'
            if isinstance(e, requests.exceptions.ReadTimeout):
                etype_msg = 'Read timeout'
                # reading is slow, increase the time limit for the reading
                timeout = (timeout[0], timeout[1] + 10)
            if attempt > MAX_ATTEMPTS:
                logger.warning(
                    'Attempt {} of {} to get {} failed: {}.'
                    .format(MAX_ATTEMPTS, MAX_ATTEMPTS, service_path, etype_msg)
                )
                raise

            logger.info(
                'Attempt {} of {} to get {} failed: {}. Retrying...'
                .format(attempt, MAX_ATTEMPTS, service_path, etype_msg)
            )


def read_or_fetch(filename,
                  directory="/etc/leapp/files",
                  service=None,
                  allow_empty=False,
                  encoding='utf-8',
                  data_stream=None,
                  allow_download=True):
    """
    Return the contents of a text file or fetch them from an online service if the file does not exist.

    :param str filename: The name of the file to read or fetch.
    :param str directory: Directory that should contain the file.
    :param str service: URL to the service providing the data if the file is missing.
    :param Optional[str] with_leapp_version: Inject the given leapp version when fetching from a service.
    :param bool allow_empty: Raise an error if the resulting data are empty.
    :param str encoding: Encoding to use when decoding the raw binary data.
    :param bool allow_download: Allow the fallback to download the data file if not present.
    :returns: Text contents of the file. Text is decoded using the provided encoding.
    :rtype: str
    """
    logger = api.current_logger()
    local_path = os.path.join(directory, filename)

    # try to get the data locally
    if not os.path.exists(local_path):
        if not allow_download:
            _raise_error(local_path, "File {lp} does not exist.".format(lp=local_path))
        logger.warning("File {lp} does not exist, falling back to online service)".format(lp=local_path))
    else:
        try:
            with io.open(local_path, encoding=encoding) as f:
                data = f.read()
                if not allow_empty and not data:
                    _raise_error(local_path, "File {lp} exists but is empty".format(lp=local_path))
                logger.warning("File {lp} successfully read ({l} bytes)".format(lp=local_path, l=len(data)))
                return data
        except EnvironmentError:
            _raise_error(local_path, "File {lp} exists but couldn't be read".format(lp=local_path))
        except Exception as e:
            raise e

    # if the data is not present locally, fetch it from the online service
    service = service or get_env("LEAPP_SERVICE_HOST", default=SERVICE_HOST_DEFAULT)
    if data_stream:
        service_path = "{s}/api/pes/{stream}/{f}".format(s=service, stream=data_stream, f=filename)
    else:
        service_path = "{s}/api/pes/{f}".format(s=service, f=filename)

    proxy = get_env("LEAPP_PROXY_HOST")
    proxies = {"https": proxy} if proxy else None
    cert = ("/etc/pki/consumer/cert.pem", "/etc/pki/consumer/key.pem")
    response = None
    try:
        response = _request_data(service_path, cert=cert, proxies=proxies)
    except requests.exceptions.RequestException as e:
        logger.error(e)
        _raise_error(local_path, "Could not fetch {f} from {sp} (unreachable address).".format(
            f=filename, sp=service_path))
    # almost certainly missing certs
    except (OSError, IOError) as e:
        logger.error(e)
        _raise_error(local_path, ("Could not fetch {f} from {sp} (missing certificates). Is the machine"
                                  " registered?".format(f=filename, sp=service_path)))
    if response.status_code != 200:
        _raise_error(local_path, "Could not fetch {f} from {sp} (error code: {e}).".format(
            f=filename, sp=service_path, e=response.status_code))

    if not allow_empty and not response.content:
        _raise_error(local_path, "File {lp} successfully retrieved but it's empty".format(lp=local_path))
    logger.warning("File {sp} successfully retrieved and read ({l} bytes)".format(
        sp=service_path, l=len(response.content)))

    return response.content.decode(encoding)


def load_data_asset(actor_requesting_asset,
                    asset_filename,
                    asset_fulltext_name,
                    docs_url,
                    docs_title):
    """
    Load the content of the data asset with given asset_filename
    and produce :class:`leapp.model.ConsumedDataAsset` message.

    :param Actor actor_requesting_asset: The actor instance requesting the asset file. It is necessary for the actor
                                         to be able to produce ConsumedDataAsset message in order for leapp to be able
                                         to uniformly report assets with incorrect versions.
    :param str asset_filename: The file name of the asset to load.
    :param str asset_fulltext_name: A human readable asset name to display in error messages.
    :param str docs_url: Docs url to provide if an asset is malformed or outdated.
    :param str docs_title: Title of the documentation to where `docs_url` points to.
    :returns: A dict with asset contents (a parsed JSON), or None if the asset was outdated.
    :raises StopActorExecutionError: In following cases:
        * ConsumedDataAsset is not specified in the produces tuple of the actor_requesting_asset actor
        * The content of the required data file is not valid JSON format
        * The required data cannot be obtained (e.g. due to missing file)
    """

    # Check that the actor that is attempting to obtain the asset meets the contract to call this function
    if models.ConsumedDataAsset not in actor_requesting_asset.produces:
        raise StopActorExecutionError('The supplied `actor_requesting_asset` does not produce ConsumedDataAsset.')

    if docs_url:
        error_hint = {'hint': ('Read documentation at the following link for more information about how to retrieve '
                               'the valid file: {0}'.format(docs_url))}
    else:
        error_hint = {'hint': _get_hint(os.path.join('/etc/leapp/files', asset_filename))}

    data_stream_id = get_consumed_data_stream_id()
    data_stream_major = data_stream_id.split('.', 1)[0]
    api.current_logger().info(
        'Attempting to load the asset {0} (data_stream={1})'.format(asset_filename, data_stream_id)
    )

    try:
        # The asset family ID has the form (major, minor), include only `major` in the URL
        raw_asset_contents = read_or_fetch(asset_filename, data_stream=data_stream_major, allow_download=False)
        asset_contents = json.loads(raw_asset_contents)
    except ValueError:
        msg = 'The {0} file (at {1}) does not contain a valid JSON object.'.format(asset_fulltext_name, asset_filename)
        raise StopActorExecutionError(msg, details=error_hint)

    if not isinstance(asset_contents, dict):
        # Should be unlikely
        msg = 'The {0} file (at {1}) is invalid - it does not contain a JSON object at the topmost level.'
        raise StopActorExecutionError(msg.format(asset_fulltext_name, asset_filename), details=error_hint)

    provided_data_streams = asset_contents.get(ASSET_PROVIDED_DATA_STREAMS_FIELD)
    if provided_data_streams and not isinstance(provided_data_streams, list):
        provided_data_streams = []  # The asset will be later reported as malformed

    api.produce(models.ConsumedDataAsset(filename=asset_filename,
                                         fulltext_name=asset_fulltext_name,
                                         docs_url=docs_url,
                                         docs_title=docs_title,
                                         provided_data_streams=provided_data_streams))

    return asset_contents
